@@ -1,4 +1,4 @@
-# Copyright 2015-2016 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -56,6 +56,7 @@ def Reformat(uwlines, verify=False):
 
     indent_amt = indent_width * uwline.depth
     state = format_decision_state.FormatDecisionState(uwline, indent_amt)
+    state.MoveStateToNextToken()
 
     if not uwline.disable:
       if uwline.first.is_comment:
@@ -74,8 +75,8 @@ def Reformat(uwlines, verify=False):
       _RetainHorizontalSpacing(uwline)
       _RetainVerticalSpacing(uwline, prev_uwline)
       _EmitLineUnformatted(state)
-    elif (_CanPlaceOnSingleLine(uwline) and not any(tok.must_split
-                                                    for tok in uwline.tokens)):
+    elif _CanPlaceOnSingleLine(uwline) and not any(tok.must_split
+                                                   for tok in uwline.tokens):
       # The unwrapped line fits on one line.
       while state.next_token:
         state.AddTokenToState(newline=False, dry_run=False)
@@ -84,6 +85,7 @@ def Reformat(uwlines, verify=False):
         # Failsafe mode. If there isn't a solution to the line, then just emit
         # it as is.
         state = format_decision_state.FormatDecisionState(uwline, indent_amt)
+        state.MoveStateToNextToken()
         _RetainHorizontalSpacing(uwline)
         _RetainVerticalSpacing(uwline, prev_uwline)
         _EmitLineUnformatted(state)
@@ -128,6 +130,9 @@ def _RetainVerticalSpacingBetweenTokens(cur_tok, prev_tok):
   else:
     cur_lineno = cur_tok.lineno
 
+  if prev_tok.value.endswith('\\'):
+    prev_lineno = prev_lineno + prev_tok.value.count('\n')
+
   cur_tok.AdjustNewlinesBefore(cur_lineno - prev_lineno)
 
 
@@ -219,8 +224,7 @@ def _CanPlaceOnSingleLine(uwline):
   indent_amt = style.Get('INDENT_WIDTH') * uwline.depth
   last = uwline.last
   last_index = -1
-  if last.is_comment and re.search(r'^#+\s+pylint:',
-                                   last.value.strip(), re.IGNORECASE):
+  if last.is_pylint_comment:
     last = last.previous_token
     last_index = -2
   if last is None:
@@ -230,6 +234,7 @@ def _CanPlaceOnSingleLine(uwline):
 
 
 def _FormatFinalLines(final_lines, verify):
+  """Compose the final output from the finalized lines."""
   formatted_code = []
   for line in final_lines:
     formatted_line = []
@@ -270,8 +275,9 @@ class _StateNode(object):
     self.previous = previous
 
   def __repr__(self):  # pragma: no cover
-    return 'StateNode(state=[\n{0}\n], newline={1})'.format(self.state,
-                                                            self.newline)
+    return 'StateNode(state=[\n{0}\n], newline={1})'.format(
+        self.state, self.newline)
+
 
 # A tuple of (penalty, count) that is used to prioritize the BFS. In case of
 # equal penalties, we prefer states that were inserted first. During state
@@ -309,7 +315,6 @@ def _AnalyzeSolutionSpace(initial_state):
   heapq.heappush(p_queue, _QueueItem(_OrderedPenalty(0, count), node))
 
   count += 1
-  prev_penalty = 0
   while p_queue:
     item = p_queue[0]
     penalty = item.ordered_penalty.penalty
@@ -324,7 +329,6 @@ def _AnalyzeSolutionSpace(initial_state):
     if node.state in seen:
       continue
 
-    prev_penalty = penalty
     seen.add(node.state)
 
     # FIXME(morbo): Add a 'decision' element?
@@ -357,10 +361,10 @@ def _AddNextStateToQueue(penalty, previous_node, newline, count, p_queue):
   Returns:
     The updated number of elements in the queue.
   """
-  if newline and not previous_node.state.CanSplit():
+  must_split = previous_node.state.MustSplit()
+  if newline and not previous_node.state.CanSplit(must_split):
     # Don't add a newline if the token cannot be split.
     return count
-  must_split = previous_node.state.MustSplit()
   if not newline and must_split:
     # Don't add a token we must split but where we aren't splitting.
     return count
@@ -444,8 +448,11 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline,
     return 0
 
   if first_token.is_docstring:
+    if (prev_uwline.first.value == 'class' and
+        style.Get('BLANK_LINE_BEFORE_CLASS_DOCSTRING')):
+      # Enforce a blank line before a class's docstring.
+      return ONE_BLANK_LINE
     # The docstring shouldn't have a newline before it.
-    # TODO(morbo): Add a knob to adjust this.
     return NO_BLANK_LINES
 
   prev_last_token = prev_uwline.last
@@ -466,7 +473,8 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline,
     if not indent_depth:
       # This is a top-level class or function.
       is_inline_comment = prev_last_token.whitespace_prefix.count('\n') == 0
-      if prev_last_token.is_comment and not is_inline_comment:
+      if (not prev_uwline.disable and prev_last_token.is_comment and
+          not is_inline_comment):
         # This token follows a non-inline comment.
         if _NoBlankLinesBeforeCurrentToken(prev_last_token.value, first_token,
                                            prev_last_token):
@@ -482,9 +490,8 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline,
           else:
             prev_last_token.AdjustNewlinesBefore(TWO_BLANK_LINES)
           if first_token.newlines is not None:
-            pytree_utils.SetNodeAnnotation(first_token.node,
-                                           pytree_utils.Annotation.NEWLINES,
-                                           None)
+            pytree_utils.SetNodeAnnotation(
+                first_token.node, pytree_utils.Annotation.NEWLINES, None)
           return NO_BLANK_LINES
     elif prev_uwline.first.value in {'class', 'def', 'async'}:
       if not style.Get('BLANK_LINE_BEFORE_NESTED_CLASS_OR_DEF'):
