@@ -1,4 +1,4 @@
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -151,6 +151,36 @@ class TestsForPython3Code(yapf_test_helper.YAPFTest):
     finally:
       style.SetGlobalStyle(style.CreatePEP8Style())
 
+  def testTypeHint(self):
+    unformatted_code = textwrap.dedent("""\
+        def foo(x: int=42):
+            pass
+
+
+        def foo2(x: 'int' =42):
+            pass
+        """)
+    expected_formatted_code = textwrap.dedent("""\
+        def foo(x: int=42):
+            pass
+
+
+        def foo2(x: 'int'=42):
+            pass
+        """)
+    uwlines = yapf_test_helper.ParseAndUnwrap(unformatted_code)
+    self.assertCodeEqual(expected_formatted_code, reformatter.Reformat(uwlines))
+
+  def testMatrixMultiplication(self):
+    unformatted_code = textwrap.dedent("""\
+        a=b@c
+        """)
+    expected_formatted_code = textwrap.dedent("""\
+        a = b @ c
+        """)
+    uwlines = yapf_test_helper.ParseAndUnwrap(unformatted_code)
+    self.assertCodeEqual(expected_formatted_code, reformatter.Reformat(uwlines))
+
   def testAsyncWithPrecedingComment(self):
     if sys.version_info[1] < 5:
       return
